/*
 * TransparentBackground.java
 *
 * Creado el 1 de marzo de 2006, 13:03
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.mhpsc.ventanastransparentes;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.Date;
import javax.swing.JComponent;
import java.awt.Image;
import javax.swing.JLabel;
import javax.swing.JWindow;

/**
 *
 * @author avelino
 */
public class TransparentBackground extends JComponent implements ComponentListener, WindowFocusListener, Runnable, MouseListener {
    private Window frame; 
    private Image background;
    private long lastupdate = 0;
    public boolean refreshRequested = true;

    public TransparentBackground(Window frame) {
        this.frame = frame;
        this.updateBackground();
        this.frame.addComponentListener(this); 
        this.frame.addWindowFocusListener(this);
        new Thread(this).start();
    }

    public void updateBackground( ) {
        try {
            Robot rbt = new Robot();
            Toolkit tk = Toolkit.getDefaultToolkit();
            Dimension dim = tk.getScreenSize();
            background = rbt.createScreenCapture(new Rectangle(0, 0, (int) dim.getWidth(), (int) dim.getHeight()));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void paintComponent(Graphics g) {
        Point pos = this.getLocationOnScreen( );
        Point offset = new Point(-pos.x, -pos.y);
        g.drawImage(background, offset.x, offset.y, null);
    }
    
    public static void main(String[] args) {
        JLabel hola = new JLabel("hola");
        JWindow w = new JWindow();
        TransparentBackground bg = new TransparentBackground(w);
        w.setSize(400, 300);
        w.show();
        w.getContentPane().add(hola);
        w.getContentPane().add(bg);
        bg.setBounds(0, 0, 400, 300);
        hola.setBounds(0, 0, 200, 200);
        hola.addMouseListener(bg);
    }

   public void refresh() {
        if (this.frame.isVisible()) {
            this.repaint();
            this.refreshRequested = true;
            this.lastupdate = new Date().getTime();
        }
    }
   
    // ComponentListener
    public void componentResized(ComponentEvent e) {
        this.repaint();
    }

    public void componentMoved(ComponentEvent e) {
        this.repaint();
    }

    public void componentShown(ComponentEvent e) {
        this.repaint();
    }

    public void componentHidden(ComponentEvent e) {
    }

    // WindowFocusListener
    public void windowGainedFocus(WindowEvent e) {
        this.refresh();
    }

    public void windowLostFocus(WindowEvent e) {
        this.refresh();
    }

    // Runnable
    public void run() {
        try {
            while(true) {
                Thread.sleep(100);
                //System.out.flush();
                long now = new Date().getTime();
                if (this.refreshRequested && ((now - this.lastupdate) > 1000)) {
                    if (this.frame.isVisible()) {
                        //System.out.println("Hola 2");
                        Point location = this.frame.getLocation();
                        this.frame.hide();
                        this.updateBackground();
                        this.frame.show();
                        this.frame.setLocation(location);
                        this.refresh();
                    }
                    this.lastupdate = now;
                    this.refreshRequested = false;
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } 
    }

    // MouseListener
    public void mouseClicked(MouseEvent e) {
        System.exit(0);
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }
}
